from typing import List, NamedTuple, Optional, Tuple

import _mecab


class Feature(NamedTuple):
    pos: str
    semantic: str
    has_jongseong: bool
    reading: str
    type: str
    start_pos: str
    end_pos: str
    exprssion: str


def _create_lattice(sentence: str) -> _mecab.Lattice:
    lattice = _mecab.Lattice()
    lattice.add_request_type(_mecab.MECAB_ALLOCATE_SENTENCE)  # Required
    lattice.set_sentence(sentence)

    return lattice


def _extract_feature(node: _mecab.Node) -> Feature:
    # Reference:
    # - http://taku910.github.io/mecab/learn.html
    # - https://docs.google.com/spreadsheets/d/1-9blXKjtjeKZqsf4NzHeYJCrr49-nXeRF6D80udfcwY
    # - https://bitbucket.org/eunjeon/mecab-ko-dic/src/master/utils/dictionary/lexicon.py

    # feature = <pos>,<semantic>,<has_jongseong>,<reading>,<type>,<start_pos>,<end_pos>,<expression>
    values = node.feature.split(",")
    assert len(values) == 8

    values = [value if value != "*" else None for value in values]
    feature = dict(zip(Feature._fields, values))
    feature["has_jongseong"] = {"T": True, "F": False}.get(feature["has_jongseong"])

    return Feature(**feature)


class MeCabError(Exception):
    pass


class MeCab:  # APIs are inspried by KoNLPy
    def __init__(self, dictionary_path: Optional[str] = None):
        arguments = []
        if dictionary_path is not None:
            arguments.append(f"-d {dictionary_path}")

        self.tagger = _mecab.Tagger(" ".join(arguments))

    def parse(self, sentence: str) -> List[Tuple[str, Feature]]:
        lattice = _create_lattice(sentence)
        if not self.tagger.parse(lattice):
            raise MeCabError(self.tagger.what())

        return [(node.surface, _extract_feature(node)) for node in lattice]

    def pos(self, sentence: str) -> List[Tuple[str, str]]:
        return [(surface, feature.pos) for surface, feature in self.parse(sentence)]

    def morphs(self, sentence: str) -> List[str]:
        return [surface for surface, _ in self.parse(sentence)]

    def nouns(self, sentence: str) -> List[str]:
        return [
            surface
            for surface, feature in self.parse(sentence)
            if feature.pos.startswith("N")
        ]



import os
import shutil
import site
import subprocess
import sys
from glob import glob
from pathlib import Path

from pybind11.setup_helpers import Pybind11Extension, build_ext
from setuptools import find_packages, setup

prefix_paths = [
    Path(sys.prefix),
    Path(site.getuserbase()),
    Path.home() / ".local",
]


class EnsureMeCabThenBuild(Pybind11Extension):
    def __init__(self, *args, **kwargs):
        self._configure_path()

        if not self._is_mecab_installed():
            self._install_mecab()

        kwargs["include_dirs"] = [self._get_mecab_include_directory()]
        kwargs["library_dirs"] = [self._get_mecab_library_directory()]
        kwargs["libraries"] = ["mecab"]
        kwargs["runtime_library_dirs"] = [self._get_mecab_library_directory()]
        super().__init__(*args, **kwargs)

    def _configure_path(self):
        for path in prefix_paths:
            os.environ["PATH"] += f"{os.pathsep}{path.absolute() / 'bin'}"

    def _is_mecab_installed(self) -> bool:
        return shutil.which("mecab") is not None

    def _install_mecab(self):
        prefix_path = self._guess_prefix()

        setup_path = Path(__file__).parent.absolute()
        scripts_path = setup_path / "scripts"
        install_script_path = scripts_path / "install-mecab-ko.py"
        subprocess.check_call(
            [
                sys.executable,
                str(install_script_path),
                "--prefix",
                str(prefix_path.absolute()),
            ]
        )

    def _get_mecab_include_directory(self) -> str:
        return subprocess.check_output(
            ["mecab-config", "--inc-dir"], encoding="utf-8"
        ).strip()

    def _get_mecab_library_directory(self) -> str:
        return subprocess.check_output(
            ["mecab-config", "--libs-only-L"], encoding="utf-8"
        ).strip()

    def _is_writable(self, path: Path) -> bool:
        # If path is not directory, find closest parent direcotry
        while not path.is_dir():
            parent = path.parent
            if path == parent:
                break
            path = parent

        return os.access(path, os.W_OK)

    def _guess_prefix(self) -> Path:
        for path in prefix_paths:
            if self._is_writable(path):
                return path

        raise RuntimeError("All prefix candidates are not writable")


with open("README.md", "r", encoding="utf-8") as input_file:
    long_description = input_file.read()


setup(
    name="python-mecab-ko",
    version="1.1.0",
    url="https://github.com/jonghwanhyeon/python-mecab-ko",
    author="Jonghwan Hyeon",
    author_email="hyeon0145@gmail.com",
    description="A python binding for mecab-ko",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="BSD",
    keywords="mecab mecab-ko",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: Korean",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Text Processing",
        "Topic :: Text Processing :: Linguistic",
    ],
    zip_safe=False,
    python_requires=">=3.7",
    packages=find_packages(),
    data_files=[("scripts", ["scripts/install-mecab-ko.py"])],
    cmdclass={"build_ext": build_ext},
    ext_modules=[
        EnsureMeCabThenBuild(
            name="_mecab",
            sources=sorted(glob("mecab/pybind/**/*.cpp", recursive=True)),
        ),
    ],
)



import argparse
import os
import subprocess
import sys
import time
import urllib.request
from contextlib import contextmanager
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Dict
from urllib.parse import urlparse

MECAB_KO_URL = "https://bitbucket.org/eunjeon/mecab-ko/downloads/mecab-{mecab_version}-ko-{mecab_ko_version}.tar.gz"
MECAB_KO_DIC_URL = "https://bitbucket.org/eunjeon/mecab-ko-dic/downloads/mecab-ko-dic-{mecab_ko_dic_version}.tar.gz"


def parse_arguments() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument("--prefix", required=True)
    parser.add_argument("--mecab_version", default="0.996")
    parser.add_argument("--mecab_ko_version", default="0.9.2")
    parser.add_argument("--mecab_ko_dic_version", default="2.1.1-20180720")

    return parser.parse_args()


@contextmanager
def change_directory(directory: str):
    original = os.path.abspath(os.getcwd())

    os.chdir(directory)
    yield

    os.chdir(original)


def path_of(filename: str) -> str:
    for path, _, filenames in os.walk(os.getcwd()):
        if filename in filenames:
            return path

    raise ValueError("File {} not found".format(filename))


def retrieve(url: str, filename: str):
    print("Downloading", url, file=sys.stderr)
    response = urllib.request.urlopen(url)
    content_length = response.getheader("Content-Length")
    if content_length is not None:
        content_length = int(content_length)

    with open(filename, "wb") as output_file:
        downloaded = 0
        progress_time = time.time()
        while True:
            block = response.read(8192)
            if not block:
                break

            output_file.write(block)
            downloaded += len(block)

            # Report progress every 0.5 seconds
            interval = time.time() - progress_time
            if interval > 0.5:
                if content_length is not None:
                    print(
                        f"> {downloaded} bytes / {content_length} bytes "
                        f"({(downloaded / content_length) * 100:.2f}%)",
                        end="\r",
                        file=sys.stderr,
                    )
                progress_time = time.time()
        print(file=sys.stderr)


def install(url: str, *args, environment: Dict[str, str] = None):
    def download(url: str):
        components = urlparse(url)
        filename = os.path.basename(components.path)

        retrieve(url, filename)
        subprocess.run(["tar", "-xzf", filename], check=True)

    def configure(*args):
        with change_directory(path_of("configure")):
            try:
                subprocess.run(["./autogen.sh"])
            except:
                pass

            subprocess.run(["./configure", *args], check=True)

    def make():
        with change_directory(path_of("Makefile")):
            subprocess.run(["make"], check=True, env=environment)
            subprocess.run(["make", "install"], check=True, env=environment)

    with TemporaryDirectory() as directory:
        with change_directory(directory):
            download(url)
            configure(*args)
            make()


if __name__ == "__main__":
    arguments = parse_arguments()

    prefix_path = Path(arguments.prefix)
    prefix_path.mkdir(parents=True, exist_ok=True)

    print("Installing mecab-ko...", file=sys.stderr)
    install(
        MECAB_KO_URL.format(
            mecab_version=arguments.mecab_version,
            mecab_ko_version=arguments.mecab_ko_version,
        ),
        f"--prefix={prefix_path}",
        "--enable-utf8-only",
    )

    print("Installing mecab-ko-dic...", file=sys.stderr)
    install(
        MECAB_KO_DIC_URL.format(mecab_ko_dic_version=arguments.mecab_ko_dic_version),
        f"--prefix={prefix_path}",
        "--with-charset=utf8",
        f"--with-mecab-config={prefix_path / 'bin' / 'mecab-config'}",
        environment={
            "LD_LIBRARY_PATH": f"{prefix_path / 'lib'}",
        },
    )
